from typing import Optional, Literal

import requests

from openkosmos_ai.common.model import AIBaseConfig
from openkosmos_ai.nodes.base_node import BaseStateNode


class ApiConfig(AIBaseConfig):
    base_url: str
    api_key: Optional[str] = None
    auth_mode: Literal["totp", "api_key"] = "api_key"


class ApiNode(BaseStateNode[ApiConfig]):
    def __init__(self, config: ApiConfig):
        super().__init__(config)
        self.api_key = None
        if config.auth_mode == "api_key":
            self.api_key = config.api_key

    def auth_totp(self, user_code: str, totp_code: str):
        api_key = self.auth_by_totp(user_code, totp_code)
        self.api_key = api_key
        return api_key

    def auth(self, api_key: str):
        self.api_key = api_key
        return self.api_key

    def get_api_key(self, runtime_api_key=None):
        if runtime_api_key is None:
            return self.api_key
        else:
            return runtime_api_key

    def auth_by_totp(self):
        return self.api_key

    def get(self, path: str, api_key=None):
        return requests.get(self.config().base_url + path, headers={
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.get_api_key(api_key)}"
        }).json()

    def delete(self, path: str, api_key=None):
        return requests.delete(self.config().base_url + path, headers={
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.get_api_key(api_key)}"
        }).json()

    def post(self, path: str, form=None, json=None, api_key=None, files=None):
        header = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.get_api_key(api_key)}"
        }
        if form is not None:
            header["Content-Type"] = "multipart/form-data"
            return requests.post(self.config().base_url + path, headers=header, data=form).json()
        if json is not None:
            return requests.post(self.config().base_url + path, headers=header, json=json).json()
        if files is not None:
            del header["Content-Type"]
            return requests.post(self.config().base_url + path, headers=header, files=files).json()
        else:
            return requests.post(self.config().base_url + path, headers=header).json()



import requests

from openkosmos_ai.common.model import AIBaseConfig
from openkosmos_ai.nodes.base_node import BaseStateNode


class RAGFlowClientConfig(AIBaseConfig):
    base_url: str
    api_key: str


class RAGFlowClientNode(BaseStateNode[RAGFlowClientConfig]):
    def __init__(self, config: RAGFlowClientConfig):
        super().__init__(config)
        self.api_prefix = "/api/v1"
        self.auth_header = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {config.api_key}"
        }

    def list_datasets(self, page=1, page_size=1000, dataset_id: str = None, orderby="create_time",
                      desc="true",
                      dataset_name=None):
        id_str = "" if dataset_id is None else f"&id={dataset_id}"
        name_str = "" if dataset_name is None else f"&name={dataset_name}"
        return requests.get(
            self.config().base_url + f"{self.api_prefix}/datasets?page={page}&page_size={page_size}&orderby={orderby}&desc={desc}{name_str}{id_str}",
            headers=self.auth_header).json()

    def list_chunks(self, dataset_id: str, document_id: str, page=1, page_size=1000):
        return requests.get(
            (
                f"{self.config().base_url}{self.api_prefix}"
                f"/datasets/{dataset_id}/documents/{document_id}/chunks"
                f"?page={page}&page_size={page_size}"
            ),
            headers=self.auth_header).json()

    def list_documents(self, dataset_id: str, document_id: str = None, page=1, page_size=1000):
        return requests.get(
            (
                f"{self.config().base_url}{self.api_prefix}"
                f"/datasets/{dataset_id}/documents"
                f"?page={page}&page_size={page_size}{"" if document_id is None else f"&id={document_id}"}"
            ),
            headers=self.auth_header).json()

    def download_document(self, dataset_id: str, document_id: str):
        return requests.get(
            (
                f"{self.config().base_url}{self.api_prefix}"
                f"/datasets/{dataset_id}/documents/{document_id}"
            ),
            headers=self.auth_header).content

    def list_chunks(self, dataset_id: str, document_id: str, page=1, page_size=1000):
        return requests.get(
            (
                f"{self.config().base_url}{self.api_prefix}"
                f"/datasets/{dataset_id}/documents/{document_id}/chunks"
                f"?page={page}&page_size={page_size}"
            ),
            headers=self.auth_header).json()

    def retrieve_chunks(self, question: str, dataset_ids: list[str], top_k=1):
        return requests.post(
            (
                f"{self.config().base_url}{self.api_prefix}"
                "/retrieval"
            ),
            headers=self.auth_header,
            json={
                "question": question,
                "dataset_ids": dataset_ids,
                "top_k": top_k
            }).json()



from typing import Optional, List

import requests

from openkosmos_ai.common.model import AIBaseConfig
from openkosmos_ai.nodes.base_node import BaseStateNode


class XinferenceClientConfig(AIBaseConfig):
    base_url: str
    api_key: str
    rerank_model: Optional[str] = "bge-reranker-v2-m3"
    embed_model: Optional[str] = "bge-m3"


class XinferenceClientNode(BaseStateNode[XinferenceClientConfig]):
    def __init__(self, config: XinferenceClientConfig):
        super().__init__(config)

    def rerank(self, documents: List[str], query: str, top_n=3):
        return requests.post(self.config().base_url + "/v1/rerank",
                             json={
                                 "model": self.config().rerank_model,
                                 "query": query,
                                 "documents": documents,
                                 "top_n": top_n
                             }).json()["results"]

    def embed(self, input: str):
        return requests.post(self.config().base_url + "/v1/embeddings",
                             json={
                                 "model": self.config().embed_model,
                                 "input": input,
                                 "encoding_format": "float"
                             }).json()



from typing import Generic, Type

from openkosmos_ai.common.ai_template_loader import AITemplateLoader, AITemplateConfig, PROMPT_ROLE, \
    TEMPLATE_SOURCE
from openkosmos_ai.common.model import AI_CONFIG_T
from openkosmos_core.config import Config


class BaseStateNode(Generic[AI_CONFIG_T]):

    def __init__(self, config=None):
        self._config = config
        c_instance = Config.instance()
        if c_instance is None:
            self._default_template_loader = AITemplateLoader()
        else:
            self._default_template_loader = AITemplateLoader(c_instance.get_template_config())
        self._config_template_loader = None

    def config_template_loader(self, config: TEMPLATE_SOURCE | AITemplateConfig):
        self._config_template_loader = AITemplateLoader(config)
        return self

    def config(self) -> Type[AI_CONFIG_T]:
        return self._config

    def get_prompt(self, name: str, role: PROMPT_ROLE = "system", use_default=False):
        if use_default or self._config_template_loader is None:
            return self._default_template_loader.get_prompt(name=name, role=role)
        else:
            return self._config_template_loader.get_prompt(name=name, role=role)



from datetime import datetime

from langchain_core.messages import AIMessage, HumanMessage, SystemMessage
from langchain_core.runnables.graph import CurveStyle
from langfuse.langchain import CallbackHandler
from langgraph.graph.state import CompiledStateGraph


def create_tool_message(messages, content="done", tool_call_index=0, tool_call_message_index=-1):
    tool_calls = messages[tool_call_message_index].tool_calls
    return {"role": "tool", "content": content, "tool_call_id": tool_calls[tool_call_index]['id']}


def create_ai_message(content: str | list[str | dict] | None = None):
    return AIMessage(content=content)


def create_user_message(content: str | list[str | dict] | None = None):
    return HumanMessage(content=content)


def create_system_message(content: str | list[str | dict] | None = None):
    return SystemMessage(content=content)


def dump_flow_result(result, user_content_block=False):
    if isinstance(result, dict):
        for m in result["messages"]:
            if user_content_block:
                for content_blocks in m.content_blocks:
                    print("---------------------------------------------------")
                    print(content_blocks)
            else:
                m.pretty_print()
    else:
        for e in result:
            if isinstance(e, tuple):
                e[0].pretty_print()
            else:
                messages = e.get("messages")
                if messages is not None:
                    messages[-1].pretty_print()


def create_run_config(run_name: str, thread_id_pattern="%Y%m%d-%H%M%S", recursion_limit=13, user_id="dev",
                      with_tracing=True):
    thread_id = datetime.now().strftime(thread_id_pattern)

    config = {
        "configurable": {"thread_id": thread_id},
        "recursion_limit": recursion_limit,
        "run_name": run_name,

    }

    if with_tracing:
        config["callbacks"] = [CallbackHandler()]
        config["metadata"] = {
            "langfuse_session_id": thread_id,
            "langfuse_user_id": user_id
        }

    return config


def save_mermaid(app: CompiledStateGraph, save_dir="./_data", xray=True, curve_style=CurveStyle.BASIS):
    with open(f"{save_dir}/{app.name}.mmd", "w", encoding="utf-8") as mmd_file:
        mmd_file.write(app.get_graph(xray=xray).draw_mermaid(curve_style=curve_style))



import os
from abc import abstractmethod, ABC
from typing import Literal, Optional

from langchain_core.prompts import ChatPromptTemplate
from langfuse import get_client
from openkosmos_core.common.model import BaseConfig
from openkosmos_core.templatex import TemplateLoader

TEMPLATE_SOURCE = Literal["local", "langfuse", "langsmith"]
TEMPLATE_FORMAT = Literal["f-string", "mustache", "jinja2", "templatex"]

PROMPT_ROLE = Literal["system", "ai", "human"]


class AITemplateConfig(BaseConfig, ABC):
    source: TEMPLATE_SOURCE = "local"
    format: TEMPLATE_FORMAT = "mustache"

    @abstractmethod
    def get_template(self, name: str, role: PROMPT_ROLE):
        pass

    @abstractmethod
    def save_template(self, name: str, content: str):
        pass

    @abstractmethod
    def get_template_content(self, name: str):
        pass


class LocalTemplateConfig(AITemplateConfig):
    base_dir: str = "./templates"
    ext: str = ".tpl"

    def get_local_template(self, name: str):
        with open(f"{self.base_dir}/{name}{self.ext}", "r", encoding="utf8") as template_file:
            return template_file.read()

    def get_template_content(self, name: str):
        return self.get_local_template(name)

    def get_template(self, name: str, role: PROMPT_ROLE):
        prompt = self.get_local_template(name)
        chat_prompt = ChatPromptTemplate.from_messages(
            messages=[(role, prompt)],
            template_format=self.format)

        return chat_prompt

    def save_template(self, name: str, content: str):
        os.makedirs(f"{self.base_dir}/{name[0:name.rindex("/")]}", exist_ok=True)
        with open(f"{self.base_dir}/{name}{self.ext}", "w",
                  encoding="utf-8") as output_file:
            output_file.write(content)


class LangfuseTemplateConfig(AITemplateConfig):
    type: Literal["chat", "text"] = "text"
    label: Optional[str] = "production"
    cache: Optional[int] = 86400
    version: Optional[int] = None

    def get_langfuse_template(self, name: str):
        return get_client().get_prompt(name, type=self.type, label=self.label,
                                       cache_ttl_seconds=self.cache,
                                       version=self.version)

    def get_template_content(self, name: str):
        return self.get_langfuse_template(name).prompt

    def get_template(self, name: str, role: PROMPT_ROLE):
        langfuse_prompt = self.get_langfuse_template(name)
        chat_prompt = ChatPromptTemplate.from_messages(
            messages=[(role, langfuse_prompt.prompt)],
            template_format=self.format
        )
        chat_prompt.metadata = {"langfuse_prompt": langfuse_prompt}

        return chat_prompt

    def save_template(self, name: str, content: str):
        get_client().create_prompt(name=name, prompt=content, type=self.type, labels=[self.label])


class LangsmithTemplateConfig(AITemplateConfig, ABC):
    pass


class AITemplateLoader(TemplateLoader):
    DEFAULT_TEMPLATE_SOURCE: TEMPLATE_SOURCE = "local"

    def __init__(self, config: TEMPLATE_SOURCE | AITemplateConfig = DEFAULT_TEMPLATE_SOURCE):
        if isinstance(config, str):
            match config:
                case "local":
                    self._config = LocalTemplateConfig()
                case "langfuse":
                    self._config = LangfuseTemplateConfig()
                case "langsmith":
                    self._config = LangsmithTemplateConfig()
                case _:
                    raise NotImplementedError(config)
        else:
            self._config = config

    def config(self) -> AITemplateConfig:
        return self._config

    def get_prompt(self, name: str, role: PROMPT_ROLE = "system") -> ChatPromptTemplate:
        return self.config().get_template(name, role)

    def transfer(self, name: str, target: AITemplateConfig):
        target.save_template(name, self.config().get_template_content(name))



from openkosmos_ai.common.model import AIBaseConfig
from openkosmos_ai.nodes.base_node import BaseStateNode
from openkosmos_ai.nodes.openai_client import OpenAIClientConfig


class SkillConfig(AIBaseConfig):
    openai: OpenAIClientConfig


class SkillNode(BaseStateNode[SkillConfig]):
    def __init__(self, config: SkillConfig):
        super().__init__(config)
